"""turtledemo/树.py

显示一个 "广度优先树" - 与经典的用
Logo 语言编写的深度优先算法绘制的树相对。

使用了:
(1) 一个树生成器，绘图是它的副作用，
生成器自身总是产生 None。
(2) 海龟克隆：在每个分叉点，当前画笔会被克隆。
所以最后会有 1024 个海龟。
"""
from turtle import Turtle, mainloop
from time import perf_counter as clock

def tree(plist, l, a, f):
    """ plist 是画笔列表
    l 是树枝长度
    a 是两个分支间夹角的一半
    f 是每个级别之间树枝缩短的倍率。"""
    if l > 3:
        lst = []
        for p in plist:
            p.forward(l)
            q = p.clone()
            p.left(a)
            q.right(a)
            lst.append(p)
            lst.append(q)
        for x in tree(lst, l*f, a, f):
            yield None

def maketree():
    p = Turtle()
    p.setundobuffer(None)
    p.hideturtle()
    p.speed(0)
    p.getscreen().tracer(30,0)
    p.left(90)
    p.penup()
    p.forward(-210)
    p.pendown()
    t = tree([p], 200, 65, 0.6375)
    for x in t:
        pass

def main():
    a=clock()
    maketree()
    b=clock()
    return "完成: %.2f 秒。" % (b-a)

if __name__ == "__main__":
    msg = main()
    print(msg)
    mainloop()



"""turtledemo/林登麦伊尔.py

在印度南部的泰米尔纳德邦，每天早晨，妇女们
会用米粉在门口地上绘制美丽的 "古拉姆" 图案。

这些图案可以用林登麦伊尔系统来表示，
并且很容易用 Python 和海龟画图画出来。

这里展示了两个例子:
(1) 蛇形古拉姆
(2) 克里希纳的脚链

取自 Marcia Ascher 的
《别处的数学，跨文化的思想探索》

"""
################################
# 迷你林登麦伊尔画图器
###############################

from turtle import *

def replace( seq, replacementRules, n ):
    for i in range(n):
        newseq = ""
        for element in seq:
            newseq = newseq + replacementRules.get(element,element)
        seq = newseq
    return seq

def draw( commands, rules ):
    for b in commands:
        try:
            rules[b]()
        except TypeError:
            try:
                draw(rules[b], rules)
            except:
                pass


def main():
    ################################
    # 示例 1: 蛇形古拉姆
    ################################


    def r():
        right(45)

    def l():
        left(45)

    def f():
        forward(7.5)

    snake_rules = {"-":r, "+":l, "f":f, "b":"f+f+f--f--f+f+f"}
    snake_replacementRules = {"b": "b+f+b--f--b+f+b"}
    snake_start = "b--f--b--f"

    drawing = replace(snake_start, snake_replacementRules, 3)

    reset()
    speed(3)
    tracer(1,0)
    ht()
    up()
    backward(195)
    down()
    draw(drawing, snake_rules)

    from time import sleep
    sleep(3)

    ################################
    # 示例 2: 克里希纳的脚链
    ################################

    def A():
        color("red")
        circle(10,90)

    def B():
        from math import sqrt
        color("black")
        l = 5/sqrt(2)
        forward(l)
        circle(l, 270)
        forward(l)

    def F():
        color("green")
        forward(10)

    krishna_rules = {"a":A, "b":B, "f":F}
    krishna_replacementRules = {"a" : "afbfa", "b" : "afbfbfbfa" }
    krishna_start = "fbfbfbfb"

    reset()
    speed(0)
    tracer(3,0)
    ht()
    left(45)
    drawing = replace(krishna_start, krishna_replacementRules, 3)
    draw(drawing, krishna_rules)
    tracer(1)
    return "完成!"

if __name__=='__main__':
    msg = main()
    print(msg)
    mainloop()



def patch():
    from sys import version_info, path
    from pathlib import Path

    releases = Path(__file__).resolve().parent / "releases"
    ver = (version_info.major, version_info.minor)
    main_dir = None
    max_ver = None

    for d in releases.iterdir():
        if d.is_dir():
            if d.name == "main":
                main_dir = d
            else:
                try:
                    v = tuple(map(int, d.name.split(".")))
                    if len(v) == 2 and (max_ver is None or v > max_ver):
                        max_ver = v
                except ValueError:  # 忽略无法解析的目录
                    pass

    # 如果当前Python版本大于已打包的最高版本且存在main目录 => 跳转到main
    if max_ver and ver > max_ver and main_dir:
        path.insert(0, str(main_dir))
    else:
        # 如果与当前版本号同名的目录存在 => 跳转到对应版本
        candidate = releases / f"{ver[0]}.{ver[1]}"
        if candidate.is_dir():
            path.insert(0, str(candidate))



"""turtledemo/平铺图案.py

构造两个由风筝形和飞镖形组成的非周期性
彭罗斯平铺图案，通过六个步骤逐渐扩大。

一开始分别是五个风筝形的太阳图案
和五个飞镖形的星星图案。

更多信息参见:
 https://zh.wikipedia.org/zh-cn/彭罗斯图
 -------------------------------------------
"""
from turtle import *
from math import cos, pi
from time import perf_counter as clock, sleep

f = (5**0.5-1)/2.0   # (sqrt(5)-1)/2 -- 黄金比例
d = 2 * cos(3*pi/10)

def kite(l):
    fl = f * l
    lt(36)
    fd(l)
    rt(108)
    fd(fl)
    rt(36)
    fd(fl)
    rt(108)
    fd(l)
    rt(144)

def dart(l):
    fl = f * l
    lt(36)
    fd(l)
    rt(144)
    fd(fl)
    lt(36)
    fd(fl)
    rt(144)
    fd(l)
    rt(144)

def inflatekite(l, n):
    if n == 0:
        px, py = pos()
        h, x, y = int(heading()), round(px,3), round(py,3)
        tiledict[(h,x,y)] = True
        return
    fl = f * l
    lt(36)
    inflatedart(fl, n-1)
    fd(l)
    rt(144)
    inflatekite(fl, n-1)
    lt(18)
    fd(l*d)
    rt(162)
    inflatekite(fl, n-1)
    lt(36)
    fd(l)
    rt(180)
    inflatedart(fl, n-1)
    lt(36)

def inflatedart(l, n):
    if n == 0:
        px, py = pos()
        h, x, y = int(heading()), round(px,3), round(py,3)
        tiledict[(h,x,y)] = False
        return
    fl = f * l
    inflatekite(fl, n-1)
    lt(36)
    fd(l)
    rt(180)
    inflatedart(fl, n-1)
    lt(54)
    fd(l*d)
    rt(126)
    inflatedart(fl, n-1)
    fd(l)
    rt(144)

def draw(l, n, th=2):
    clear()
    l = l * f**n
    shapesize(l/100.0, l/100.0, th)
    for k in tiledict:
        h, x, y = k
        setpos(x, y)
        setheading(h)
        if tiledict[k]:
            shape("kite")
            color("black", (0, 0.75, 0))
        else:
            shape("dart")
            color("black", (0.75, 0, 0))
        stamp()

def sun(l, n):
    for i in range(5):
        inflatekite(l, n)
        lt(72)

def star(l,n):
    for i in range(5):
        inflatedart(l, n)
        lt(72)

def makeshapes():
    tracer(0)
    begin_poly()
    kite(100)
    end_poly()
    register_shape("kite", get_poly())
    begin_poly()
    dart(100)
    end_poly()
    register_shape("dart", get_poly())
    tracer(1)

def start():
    reset()
    ht()
    pu()
    makeshapes()
    resizemode("user")

def test(l=200, n=4, fun=sun, startpos=(0,0), th=2):
    global tiledict
    goto(startpos)
    setheading(0)
    tiledict = {}
    tracer(0)
    fun(l, n)
    draw(l, n, th)
    tracer(1)
    nk = len([x for x in tiledict if tiledict[x]])
    nd = len([x for x in tiledict if not tiledict[x]])
    print("%d个风筝形和%d个飞镖形，一共%d片。" % (nk, nd, nk+nd))

def demo(fun=sun):
    start()
    for i in range(8):
        a = clock()
        test(300, i, fun)
        b = clock()
        t = b - a
        if t < 2:
            sleep(2 - t)

def main():
    #title("风筝形和飞镖形组成的彭罗斯平铺。")
    mode("logo")
    bgcolor(0.3, 0.3, 0)
    demo(sun)
    sleep(2)
    demo(star)
    pencolor("black")
    goto(0,-200)
    pencolor(0.7,0.7,1)
    write("请稍等...",
          align="center", font=('Microsoft YaHei', 36, 'bold'))
    test(600, 8, startpos=(70, 117))
    return "完成"

if __name__ == "__main__":
    msg = main()
    mainloop()



"""Define the menu contents, hotkeys, and event bindings.

There is additional configuration information in the EditorWindow class (and
subclasses): the menus are created there based on the menu_specs (class)
variable, and menus not created are silently skipped in the code here.  This
makes it possible, for example, to define a Debug menu which is only present in
the PythonShell window, and a Format menu which is only present in the Editor
windows.

"""
from importlib.util import find_spec

from idlelib.config import idleConf

#   Warning: menudefs is altered in macosx.overrideRootMenu()
#   after it is determined that an OS X Aqua Tk is in use,
#   which cannot be done until after Tk() is first called.
#   Do not alter the 'file', 'options', or 'help' cascades here
#   without altering overrideRootMenu() as well.
#       TODO: Make this more robust

menudefs = [
 # underscore prefixes character to underscore
 ('file', [
   ('新建文件(_N)', '<<open-new-window>>'),
   ('打开(_O)...', '<<open-window-from-file>>'),
   ('打开模块(_M)...', '<<open-module>>'),
   ('模块查看器(_B)', '<<open-class-browser>>'),
   ('路径查看器(_P)', '<<open-path-browser>>'),
   None,
   ('保存(_S)', '<<save-window>>'),
   ('另存为(_A)...', '<<save-window-as-file>>'),
   ('保存副本(_Y)...', '<<save-copy-of-window-as-file>>'),
   None,
   ('打印窗口(_T)', '<<print-window>>'),
   None,
   ('关闭窗口(_C)', '<<close-window>>'),
   ('退出(_X)', '<<close-all-windows>>'),
   ]),

 ('edit', [
   ('撤销(_U)', '<<undo>>'),
   ('恢复(_R)', '<<redo>>'),
   None,
   ('全选(_A)', '<<select-all>>'),
   ('剪切(_T)', '<<cut>>'),
   ('复制(_C)', '<<copy>>'),
   ('粘贴(_P)', '<<paste>>'),
   None,
   ('查找(_F)...', '<<find>>'),
   ('查找下一个(_G)', '<<find-again>>'),
   ('查找选区(_S)', '<<find-selection>>'),
   ('查找所有文件...', '<<find-in-files>>'),
   ('替换(_E)...', '<<replace>>'),
   None,
   ('跳转行号(_L)', '<<goto-line>>'),
   ('显示智能提示(_H)', '<<force-open-completions>>'),
   ('自动补全(_X)', '<<expand-word>>'),
   ('显示函数说明(_A)', '<<force-open-calltip>>'),
   ('高亮括号对(_A)', '<<flash-paren>>'),
   ]),

 ('format', [
   ('格式化(_O)', '<<format-paragraph>>'),
   ('向右缩进(_I)', '<<indent-region>>'),
   ('向左缩进(_D)', '<<dedent-region>>'),
   ('注释(_O)', '<<comment-region>>'),
   ('取消注释(_N)', '<<uncomment-region>>'),
   ('空格转制表符', '<<tabify-region>>'),
   ('制表符转空格', '<<untabify-region>>'),
   ('缩进类型', '<<toggle-tabs>>'),
   ('缩进宽度', '<<change-indentwidth>>'),
   ('清理行尾空格(_T)', '<<do-rstrip>>'),
   ]),

 ('run', [
   ('运行代码(_U)', '<<run-module>>'),
   ('自定义运行(_C)...', '<<run-custom>>'),
   ('检查代码(_H)', '<<check-module>>'),
   ('打开命令行', '<<open-python-shell>>'),
   ]),

 ('shell', [
   ('查看上次重启(_V)', '<<view-restart>>'),
   ('重启命令行(_R)', '<<restart-shell>>'),
   None,
   ('上一个历史(_P)', '<<history-previous>>'),
   ('下一个历史(_N)', '<<history-next>>'),
   None,
   ('停止运行(_I)', '<<interrupt-execution>>'),
   ]),

 ('debug', [
   ('转到对应的文件/行(_G)', '<<goto-file-line>>'),
   ('!调试器(_D)', '<<toggle-debugger>>'),
   ('堆栈调试器(_S)', '<<open-stack-viewer>>'),
   ('!自动查看堆栈(_A)', '<<toggle-jit-stack-viewer>>'),
   ]),

 ('options', [
   ('IDLE 设置(_I)', '<<open-config-dialog>>'),
   None,
   ('显示代码上文(_C)', '<<toggle-code-context>>'),
   ('显示行号(_L)', '<<toggle-line-numbers>>'),
   ('最大化窗口高度(_Z)', '<<zoom-height>>'),
   ]),

 ('window', [
   ]),

 ('help', [
   ('关于(_A)', '<<about-idle>>'),
   None,
   ('IDLE 文档(_I)', '<<help>>'),
   ('Python 文档(_D)', '<<python-docs>>'),
   ]),
]

if find_spec('turtledemo'):
    menudefs[-1][1].append(('海龟画图演示', '<<open-turtle-demo>>'))

default_keydefs = idleConf.GetCurrentKeySet()

if __name__ == '__main__':
    from unittest import main
    main('idlelib.idle_test.test_mainmenu', verbosity=2)



"""turtledemo/极简汉诺塔.py

一个极简的汉诺塔动画：
六个盘从左边柱子转移到右边柱子。

个人认为这是一个相当简洁优雅的实现，
它采用一个继承内置列表类的塔类。

盘片其实是方形的海龟，
通过 shapesize() 拉伸成长方形。
"""
from turtle import *

class Disc(Turtle):
    def __init__(self, n):
        Turtle.__init__(self, shape="square", visible=False)
        self.pu()
        self.shapesize(1.5, n*1.5, 2) # square-->rectangle
        self.fillcolor(n/6., 0, 1-n/6.)
        self.st()

class Tower(list):
    "汉诺塔，内置 list 类的子类"
    def __init__(self, x):
        "创建一个空的塔，x 是柱子的 x 坐标"
        self.x = x
    def push(self, d):
        d.setx(self.x)
        d.sety(-150+34*len(self))
        self.append(d)
    def pop(self):
        d = list.pop(self)
        d.sety(150)
        return d

def hanoi(n, from_, with_, to_):
    if n > 0:
        hanoi(n-1, from_, to_, with_)
        to_.push(from_.pop())
        hanoi(n-1, with_, from_, to_)

def play():
    onkey(None,"space")
    clear()
    try:
        hanoi(6, t1, t2, t3)
        write("点击停止按钮退出",
              align="center", font=("Microsoft YaHei", 16, "bold"))
    except Terminator:
        pass  # 用户按下停止

def main():
    global t1, t2, t3
    ht(); penup(); goto(0, -225)   # 负责写字的海龟
    t1 = Tower(-250)
    t2 = Tower(0)
    t3 = Tower(250)
    # 准备一个六层的塔
    for i in range(6,0,-1):
        t1.push(Disc(i))
    # 准备一个简单的用户界面 ;-)
    write("按下空格开始动画",
          align="center", font=("Microsoft YaHei", 16, "bold"))
    onkey(play, "space")
    listen()
    return "EVENTLOOP"

if __name__=="__main__":
    msg = main()
    print(msg)
    mainloop()



#!/usr/bin/env python3
"""     海龟画图演示套装:

           分形曲线.py

本程序绘制两种分形曲线设计:
(1) 一个希尔伯特曲线（在一个方框中）
(2) 一个科赫曲线的组合。

CurvesTurtle 类和分形曲线方法
取自 PythonCard 的海龟画图脚本。
"""
from turtle import *
from time import sleep, perf_counter as clock

class CurvesTurtle(Pen):
    # 示例来源于
    # 海龟图形学：用计算机探索数学
    # 作者 Harold Abelson 和 Andrea diSessa
    # 第96-98页
    def hilbert(self, size, level, parity):
        if level == 0:
            return
        # 旋转并绘制第一个子曲线，与大曲线的奇偶性相反
        self.left(parity * 90)
        self.hilbert(size, level - 1, -parity)
        # 连接并绘制第二个子曲线，与大曲线的奇偶性相同
        self.forward(size)
        self.right(parity * 90)
        self.hilbert(size, level - 1, parity)
        # 第三个子曲线
        self.forward(size)
        self.hilbert(size, level - 1, parity)
        # 第四个子曲线
        self.right(parity * 90)
        self.forward(size)
        self.hilbert(size, level - 1, -parity)
        # 最终需要旋转一下，让海龟面向大方块外部
        self.left(parity * 90)

    # Logo 语言视觉建模：结构化的视角
    # 作者 James Clayson
    # 科赫曲线，按照 Helge von Koch 在1904年引入的几何图形
    # 第146页
    def fractalgon(self, n, rad, lev, dir):
        import math

        # 若 dir = 1 向外转
        # 若 dir = -1 向内转
        edge = 2 * rad * math.sin(math.pi / n)
        self.pu()
        self.fd(rad)
        self.pd()
        self.rt(180 - (90 * (n - 2) / n))
        for i in range(n):
            self.fractal(edge, lev, dir)
            self.rt(360 / n)
        self.lt(180 - (90 * (n - 2) / n))
        self.pu()
        self.bk(rad)
        self.pd()

    # 第146页
    def fractal(self, dist, depth, dir):
        if depth < 1:
            self.fd(dist)
            return
        self.fractal(dist / 3, depth - 1, dir)
        self.lt(60 * dir)
        self.fractal(dist / 3, depth - 1, dir)
        self.rt(120 * dir)
        self.fractal(dist / 3, depth - 1, dir)
        self.lt(60 * dir)
        self.fractal(dist / 3, depth - 1, dir)

def main():
    ft = CurvesTurtle()

    ft.reset()
    ft.speed(0)
    ft.ht()
    ft.getscreen().tracer(1,0)
    ft.pu()

    size = 6
    ft.setpos(-33*size, -32*size)
    ft.pd()

    ta=clock()
    ft.fillcolor("red")
    ft.begin_fill()
    ft.fd(size)

    ft.hilbert(size, 6, 1)

    # frame
    ft.fd(size)
    for i in range(3):
        ft.lt(90)
        ft.fd(size*(64+i%2))
    ft.pu()
    for i in range(2):
        ft.fd(size)
        ft.rt(90)
    ft.pd()
    for i in range(4):
        ft.fd(size*(66+i%2))
        ft.rt(90)
    ft.end_fill()
    tb=clock()
    res =  "希尔伯特曲线: %.2f 秒。" % (tb-ta)

    sleep(3)

    ft.reset()
    ft.speed(0)
    ft.ht()
    ft.getscreen().tracer(1,0)

    ta=clock()
    ft.color("black", "blue")
    ft.begin_fill()
    ft.fractalgon(3, 250, 4, 1)
    ft.end_fill()
    ft.begin_fill()
    ft.color("red")
    ft.fractalgon(3, 200, 4, -1)
    ft.end_fill()
    tb=clock()
    res +=  "科赫曲线: %.2f 秒。" % (tb-ta)
    return res

if __name__  == '__main__':
    msg = main()
    print(msg)
    mainloop()



#!/usr/bin/env python3
"""     海龟画图演示套装:

             画图.py

一个简单的事件驱动的画图程序

- 左键移动海龟
- 中间改变颜色
- 右键切换提笔和落笔
（提笔就是移动的时候不画线）
（落笔就是画线）
如果落笔后移动了至少两步，在提笔后
每一步构成的多边形将被填充颜色。
 ----------------------------------
 使用三个鼠标按键点击画布进行操作。
 ----------------------------------
          点击停止按钮退出
 ----------------------------------
"""
from turtle import *

def switchupdown(x=0, y=0):
    if pen()["pendown"]:
        end_fill()
        up()
    else:
        down()
        begin_fill()

def changecolor(x=0, y=0):
    global colors
    colors = colors[1:]+colors[:1]
    color(colors[0])

def main():
    global colors
    shape("circle")
    resizemode("user")
    shapesize(.5)
    width(3)
    colors=["red", "green", "blue", "yellow"]
    color(colors[0])
    switchupdown()
    onscreenclick(goto,1)
    onscreenclick(changecolor,2)
    onscreenclick(switchupdown,3)
    return "EVENTLOOP"

if __name__ == "__main__":
    msg = main()
    print(msg)
    mainloop()



"""turtledemo/森林.py

显示一个三颗广度优先树组成的 "森林"，
类似于 树.py 中的树。
关于进一步的解释，请参考 树.py。

该演示是对 Erich Neuwirth 编写的一个类似的
Logo 编程语言程序的 "广度优先" 改编。
"""
from turtle import Turtle, colormode, tracer, mainloop
from random import randrange
from time import perf_counter as clock

def symRandom(n):
    return randrange(-n,n+1)

def randomize( branchlist, angledist, sizedist ):
    return [ (angle+symRandom(angledist),
              sizefactor*1.01**symRandom(sizedist))
                     for angle, sizefactor in branchlist ]

def randomfd( t, distance, parts, angledist ):
    for i in range(parts):
        t.left(symRandom(angledist))
        t.forward( (1.0 * distance)/parts )

def tree(tlist, size, level, widthfactor, branchlists, angledist=10, sizedist=5):
    # 使用 turtles 和 branchlists 列表，每个 turtle 对应一个 branchlist！
    if level > 0:
        lst = []
        brs = []
        for t, branchlist in list(zip(tlist,branchlists)):
            t.pensize( size * widthfactor )
            t.pencolor( 255 - (180 - 11 * level + symRandom(15)),
                        180 - 11 * level + symRandom(15),
                        0 )
            t.pendown()
            randomfd(t, size, level, angledist )
            yield 1
            for angle, sizefactor in branchlist:
                t.left(angle)
                lst.append(t.clone())
                brs.append(randomize(branchlist, angledist, sizedist))
                t.right(angle)
        for x in tree(lst, size*sizefactor, level-1, widthfactor, brs,
                      angledist, sizedist):
            yield None


def start(t,x,y):
    colormode(255)
    t.reset()
    t.speed(0)
    t.hideturtle()
    t.left(90)
    t.penup()
    t.setpos(x,y)
    t.pendown()

def doit1(level, pen):
    pen.hideturtle()
    start(pen, 20, -208)
    t = tree( [pen], 80, level, 0.1, [[ (45,0.69), (0,0.65), (-45,0.71) ]] )
    return t

def doit2(level, pen):
    pen.hideturtle()
    start(pen, -135, -130)
    t = tree( [pen], 120, level, 0.1, [[ (45,0.69), (-45,0.71) ]] )
    return t

def doit3(level, pen):
    pen.hideturtle()
    start(pen, 190, -90)
    t = tree( [pen], 100, level, 0.1, [[ (45,0.7), (0,0.72), (-45,0.65) ]] )
    return t

# 这儿有三个树生成器:
def main():
    p = Turtle()
    p.ht()
    tracer(75,0)
    u = doit1(6, Turtle(undobuffersize=1))
    s = doit2(7, Turtle(undobuffersize=1))
    t = doit3(5, Turtle(undobuffersize=1))
    a = clock()
    while True:
        done = 0
        for b in u,s,t:
            try:
                b.__next__()
            except:
                done += 1
        if done == 3:
            break

    tracer(1,10)
    b = clock()
    return "运行时间: %.2f 秒。" % (b-a)

if __name__ == '__main__':
    main()
    mainloop()



#!/usr/bin/env python3
"""     海龟画图演示套装:

           行星系统.py

引力系统模拟，采用了费曼讲座第9-8页的方法，
通过海龟图形实现。

比如：很重的中心天体、比较轻的行星、
还有特别轻的卫星！行星有圆形的轨道，
卫星有稳定围绕行星旋转的轨道。

你可以在画布的滚动条上按住左键来暂停。

"""
from turtle import Shape, Turtle, mainloop, Vec2D as Vec

G = 8

class GravSys(object):
    def __init__(self):
        self.planets = []
        self.t = 0
        self.dt = 0.01
    def init(self):
        for p in self.planets:
            p.init()
    def start(self):
        for i in range(10000):
            self.t += self.dt
            for p in self.planets:
                p.step()

class Star(Turtle):
    def __init__(self, m, x, v, gravSys, shape):
        Turtle.__init__(self, shape=shape)
        self.penup()
        self.m = m
        self.setpos(x)
        self.v = v
        gravSys.planets.append(self)
        self.gravSys = gravSys
        self.resizemode("user")
        self.pendown()
    def init(self):
        dt = self.gravSys.dt
        self.a = self.acc()
        self.v = self.v + 0.5*dt*self.a
    def acc(self):
        a = Vec(0,0)
        for planet in self.gravSys.planets:
            if planet != self:
                v = planet.pos()-self.pos()
                a += (G*planet.m/abs(v)**3)*v
        return a
    def step(self):
        dt = self.gravSys.dt
        self.setpos(self.pos() + dt*self.v)
        if self.gravSys.planets.index(self) != 0:
            self.setheading(self.towards(self.gravSys.planets[0]))
        self.a = self.acc()
        self.v = self.v + dt*self.a

## 创建黄蓝色结合的行星形状

def main():
    s = Turtle()
    s.reset()
    s.getscreen().tracer(0,0)
    s.ht()
    s.pu()
    s.fd(6)
    s.lt(90)
    s.begin_poly()
    s.circle(6, 180)
    s.end_poly()
    m1 = s.get_poly()
    s.begin_poly()
    s.circle(6,180)
    s.end_poly()
    m2 = s.get_poly()

    planetshape = Shape("compound")
    planetshape.addcomponent(m1,"orange")
    planetshape.addcomponent(m2,"blue")
    s.getscreen().register_shape("planet", planetshape)
    s.getscreen().tracer(1,0)

    ## 设置引力系统
    gs = GravSys()
    sun = Star(1000000, Vec(0,0), Vec(0,-2.5), gs, "circle")
    sun.color("yellow")
    sun.shapesize(1.8)
    sun.pu()
    earth = Star(12500, Vec(210,0), Vec(0,195), gs, "planet")
    earth.pencolor("green")
    earth.shapesize(0.8)
    moon = Star(1, Vec(220,0), Vec(0,295), gs, "planet")
    moon.pencolor("blue")
    moon.shapesize(0.5)
    gs.init()
    gs.start()
    return "完成!"

if __name__ == '__main__':
    main()
    mainloop()



#!/usr/bin/env python3
"""     海龟画图演示套装:

           字节设计.py

改编自 PythonCard 项目的海龟画图演示套装。

基于《BYTE》（字节）杂志的文章
《Logo 语言解决问题：利用海龟画图设计图案》
1982年11月，第118-134页

-------------------------------------------

由于第149行的代码

t.delay(0)

将动画延迟设为0，此动画会一行行地高速运行。
"""

from turtle import Turtle, mainloop
from time import perf_counter as clock

# 为任何需要相互了解的附加绘图例程提供封装
class Designer(Turtle):

    def design(self, homePos, scale):
        self.up()
        for i in range(5):
            self.forward(64.65 * scale)
            self.down()
            self.wheel(self.position(), scale)
            self.up()
            self.backward(64.65 * scale)
            self.right(72)
        self.up()
        self.goto(homePos)
        self.right(36)
        self.forward(24.5 * scale)
        self.right(198)
        self.down()
        self.centerpiece(46 * scale, 143.4, scale)
        self.getscreen().tracer(True)

    def wheel(self, initpos, scale):
        self.right(54)
        for i in range(4):
            self.pentpiece(initpos, scale)
        self.down()
        self.left(36)
        for i in range(5):
            self.tripiece(initpos, scale)
        self.left(36)
        for i in range(5):
            self.down()
            self.right(72)
            self.forward(28 * scale)
            self.up()
            self.backward(28 * scale)
        self.left(54)
        self.getscreen().update()

    def tripiece(self, initpos, scale):
        oldh = self.heading()
        self.down()
        self.backward(2.5 * scale)
        self.tripolyr(31.5 * scale, scale)
        self.up()
        self.goto(initpos)
        self.setheading(oldh)
        self.down()
        self.backward(2.5 * scale)
        self.tripolyl(31.5 * scale, scale)
        self.up()
        self.goto(initpos)
        self.setheading(oldh)
        self.left(72)
        self.getscreen().update()

    def pentpiece(self, initpos, scale):
        oldh = self.heading()
        self.up()
        self.forward(29 * scale)
        self.down()
        for i in range(5):
            self.forward(18 * scale)
            self.right(72)
        self.pentr(18 * scale, 75, scale)
        self.up()
        self.goto(initpos)
        self.setheading(oldh)
        self.forward(29 * scale)
        self.down()
        for i in range(5):
            self.forward(18 * scale)
            self.right(72)
        self.pentl(18 * scale, 75, scale)
        self.up()
        self.goto(initpos)
        self.setheading(oldh)
        self.left(72)
        self.getscreen().update()

    def pentl(self, side, ang, scale):
        if side < (2 * scale): return
        self.forward(side)
        self.left(ang)
        self.pentl(side - (.38 * scale), ang, scale)

    def pentr(self, side, ang, scale):
        if side < (2 * scale): return
        self.forward(side)
        self.right(ang)
        self.pentr(side - (.38 * scale), ang, scale)

    def tripolyr(self, side, scale):
        if side < (4 * scale): return
        self.forward(side)
        self.right(111)
        self.forward(side / 1.78)
        self.right(111)
        self.forward(side / 1.3)
        self.right(146)
        self.tripolyr(side * .75, scale)

    def tripolyl(self, side, scale):
        if side < (4 * scale): return
        self.forward(side)
        self.left(111)
        self.forward(side / 1.78)
        self.left(111)
        self.forward(side / 1.3)
        self.left(146)
        self.tripolyl(side * .75, scale)

    def centerpiece(self, s, a, scale):
        self.forward(s); self.left(a)
        if s < (7.5 * scale):
            return
        self.centerpiece(s - (1.2 * scale), a, scale)

def main():
    t = Designer()
    t.speed(0)
    t.hideturtle()
    t.getscreen().delay(0)
    t.getscreen().tracer(0)
    at = clock()
    t.design(t.position(), 2)
    et = clock()
    return "运行时间: %.2f 秒。" % (et-at)

if __name__ == '__main__':
    msg = main()
    print(msg)
    mainloop()



#!/usr/bin/env python3

"""
  ----------------------------------------------
      海龟画图演示 - 帮助
  ----------------------------------------------

  该文档有两部分:

  (1) 如何使用演示器
  (2) 如何添加自己的演示


  (1) 如何使用演示器.

  从演示菜单中选择一个演示脚本。
  彩色的源代码将在左侧的代码窗口中显示。
  代码不能编辑，只能查看！

  演示器窗口可以改变大小，代码和画图区中间的分割线也可以用鼠标拖动。
  文本字号可以在菜单栏中调整，也可用快捷键 Ctrl/Command+加号/减号。
  在大多数系统上，也可以把光标放在文本上，然后按 Ctrl+鼠标滚轮调整字号。

  点击开始按钮启动演示。
  点击停止按钮结束演示。
  点击清空按钮清空画图区。
  再次点击开始按钮可重新运行。

  有几个特殊的演示，比如 时钟.py，是事件驱动的。

      点击开始按钮启动演示。

      - 进入事件循环前，一切都与正常演示无异。

      - 进入事件循环后，您需要用鼠标键盘（或者演示里的自动计时器）来控制它。
      要结束演示，必须点击停止按钮。

      在事件循环运行过程中，演示菜单被禁用。

      - 必须在结束演示后，才可以重新启动它，或者选择另外的演示。

   * * * * * * * *
   在极少数情况下，演示脚本和演示器的事件可能产生冲突（它们运行在同一个进程中），
   可能导致一些奇怪的事情发生。在最坏的情况下，您需要关闭并重新打开演示器。
   * * * * * * * *


   (2) 如何添加自己的演示

   - 将演示脚本文件放在与 turtledemo/__main__.py 相同目录中。
     注意！当演示脚本被导入时，它不可以调用其他模块的函数
     （比如 sys、tkinter、turtle 等）来修改系统。全局变量应在 main() 中初始化。

   - 代码需要有一个 main() 函数，演示器将会执行这个函数（参考示例脚本）。
     它可以返回一个字符串，这个字符串会在演示结束后显示在代码窗口下方的标签里。

   - 为了在开发时可以单独运行演示脚本（比如 mydemo.py），可以在脚本的末尾加上这些代码:

    if __name__ == '__main__':
        main()
        mainloop()  # 保持窗口开启

    python -m turtledemo.mydemo  # 执行这行命令来运行演示

   - 如果演示是事件驱动的，main 函数必须返回字符串 "EVENTLOOP"，
     来告诉演示器我们的脚本仍在运行，需要让用户来停止！

     如果一个事件驱动的演示需要自动运行，比如使用 ontimer 定时器的时钟，
     以及使用递归循环的最小汉诺塔，代码应捕获用户按下停止按钮时引发的
     turtle.Terminator 异常。（画图不是事件驱动的，它只是相应鼠标点击和移动）
"""
import sys
import os

from tkinter import *
from idlelib.colorizer import ColorDelegator, color_config
from idlelib.percolator import Percolator
from idlelib.textview import view_text
from turtledemo import __doc__ as about_turtledemo

import turtle

demo_dir = os.path.dirname(os.path.abspath(__file__))
darwin = sys.platform == 'darwin'

STARTUP = 1
READY = 2
RUNNING = 3
DONE = 4
EVENTDRIVEN = 5

menufont = ("Microsoft YaHei", 12, NORMAL)
btnfont = ("Microsoft YaHei", 12, 'bold')
txtfont = ['Microsoft YaHei', 10, 'normal']

MINIMUM_FONT_SIZE = 6
MAXIMUM_FONT_SIZE = 100
font_sizes = [8, 9, 10, 11, 12, 14, 18, 20, 22, 24, 30]

def getExampleEntries():
    return [entry[:-3] for entry in os.listdir(demo_dir) if
            entry.endswith(".py") and entry[0] != '_']

help_entries = (  # (help_label,  help_doc)
    ('演示帮助', __doc__),
    ('关于演示器', about_turtledemo),
    ('关于海龟画图', turtle.__doc__),
    )


class DemoWindow(object):

    def __init__(self, filename=None):
        self.root = root = turtle._root = Tk()
        root.title('Python 海龟画图演示')
        root.wm_protocol("WM_DELETE_WINDOW", self._destroy)

        if darwin:
            import subprocess
            # Make sure we are the currently activated OS X application
            # so that our menu bar appears.
            subprocess.run(
                    [
                        'osascript',
                        '-e', 'tell application "System Events"',
                        '-e', 'set frontmost of the first process whose '
                              'unix id is {} to true'.format(os.getpid()),
                        '-e', 'end tell',
                    ],
                    stderr=subprocess.DEVNULL,
                    stdout=subprocess.DEVNULL,)

        root.grid_rowconfigure(0, weight=1)
        root.grid_columnconfigure(0, weight=1)
        root.grid_columnconfigure(1, minsize=90, weight=1)
        root.grid_columnconfigure(2, minsize=90, weight=1)
        root.grid_columnconfigure(3, minsize=90, weight=1)

        self.mBar = Menu(root, relief=RAISED, borderwidth=2)
        self.mBar.add_cascade(menu=self.makeLoadDemoMenu(self.mBar),
                              label='演示', underline=0)
        self.mBar.add_cascade(menu=self.makeFontMenu(self.mBar),
                              label='字号', underline=0)
        self.mBar.add_cascade(menu=self.makeHelpMenu(self.mBar),
                              label='帮助', underline=0)
        root['menu'] = self.mBar

        pane = PanedWindow(orient=HORIZONTAL, sashwidth=5,
                           sashrelief=SOLID, bg='#ddd')
        pane.add(self.makeTextFrame(pane))
        pane.add(self.makeGraphFrame(pane))
        pane.grid(row=0, columnspan=4, sticky='news')

        self.output_lbl = Label(root, height= 1, text=" --- ", bg="#ddf",
                                font=("Microsoft YaHei", 16, 'normal'), borderwidth=2,
                                relief=RIDGE)
        if darwin:  # Leave Mac button colors alone - #44254.
            self.start_btn = Button(root, text=" 开始 ", font=btnfont,
                                    fg='#00cc22', command=self.startDemo)
            self.stop_btn = Button(root, text=" 停止 ", font=btnfont,
                                   fg='#00cc22', command=self.stopIt)
            self.clear_btn = Button(root, text=" 清空 ", font=btnfont,
                                    fg='#00cc22', command = self.clearCanvas)
        else:
            self.start_btn = Button(root, text=" 开始 ", font=btnfont,
                                    fg="white", disabledforeground = "#fed",
                                    command=self.startDemo)
            self.stop_btn = Button(root, text=" 停止 ", font=btnfont,
                                   fg="white", disabledforeground = "#fed",
                                   command=self.stopIt)
            self.clear_btn = Button(root, text=" 清空 ", font=btnfont,
                                    fg="white", disabledforeground="#fed",
                                    command = self.clearCanvas)
        self.output_lbl.grid(row=1, column=0, sticky='news', padx=(0,5))
        self.start_btn.grid(row=1, column=1, sticky='ew')
        self.stop_btn.grid(row=1, column=2, sticky='ew')
        self.clear_btn.grid(row=1, column=3, sticky='ew')

        Percolator(self.text).insertfilter(ColorDelegator())
        self.dirty = False
        self.exitflag = False
        if filename:
            self.loadfile(filename)
        self.configGUI(DISABLED, DISABLED, DISABLED,
                       "从菜单栏选择一个演示吧", "black")
        self.state = STARTUP


    def onResize(self, event):
        cwidth = self._canvas.winfo_width()
        cheight = self._canvas.winfo_height()
        self._canvas.xview_moveto(0.5*(self.canvwidth-cwidth)/self.canvwidth)
        self._canvas.yview_moveto(0.5*(self.canvheight-cheight)/self.canvheight)

    def makeTextFrame(self, root):
        self.text_frame = text_frame = Frame(root)
        self.text = text = Text(text_frame, name='text', padx=5,
                                wrap='none', width=45)
        color_config(text)

        self.vbar = vbar = Scrollbar(text_frame, name='vbar')
        vbar['command'] = text.yview
        vbar.pack(side=LEFT, fill=Y)
        self.hbar = hbar = Scrollbar(text_frame, name='hbar', orient=HORIZONTAL)
        hbar['command'] = text.xview
        hbar.pack(side=BOTTOM, fill=X)
        text['yscrollcommand'] = vbar.set
        text['xscrollcommand'] = hbar.set

        text['font'] = tuple(txtfont)
        shortcut = 'Command' if darwin else 'Control'
        text.bind_all('<%s-minus>' % shortcut, self.decrease_size)
        text.bind_all('<%s-underscore>' % shortcut, self.decrease_size)
        text.bind_all('<%s-equal>' % shortcut, self.increase_size)
        text.bind_all('<%s-plus>' % shortcut, self.increase_size)
        text.bind('<Control-MouseWheel>', self.update_mousewheel)
        text.bind('<Control-Button-4>', self.increase_size)
        text.bind('<Control-Button-5>', self.decrease_size)

        text.pack(side=LEFT, fill=BOTH, expand=1)
        return text_frame

    def makeGraphFrame(self, root):
        turtle._Screen._root = root
        self.canvwidth = 1000
        self.canvheight = 800
        turtle._Screen._canvas = self._canvas = canvas = turtle.ScrolledCanvas(
                root, 800, 600, self.canvwidth, self.canvheight)
        canvas.adjustScrolls()
        canvas._rootwindow.bind('<Configure>', self.onResize)
        canvas._canvas['borderwidth'] = 0

        self.screen = _s_ = turtle.Screen()
        turtle.TurtleScreen.__init__(_s_, _s_._canvas)
        self.scanvas = _s_._canvas
        turtle.RawTurtle.screens = [_s_]
        return canvas

    def set_txtsize(self, size):
        txtfont[1] = size
        self.text['font'] = tuple(txtfont)
        self.output_lbl['text'] = '字号 %d' % size

    def decrease_size(self, dummy=None):
        self.set_txtsize(max(txtfont[1] - 1, MINIMUM_FONT_SIZE))
        return 'break'

    def increase_size(self, dummy=None):
        self.set_txtsize(min(txtfont[1] + 1, MAXIMUM_FONT_SIZE))
        return 'break'

    def update_mousewheel(self, event):
        # For wheel up, event.delta = 120 on Windows, -1 on darwin.
        # X-11 sends Control-Button-4 event instead.
        if (event.delta < 0) == (not darwin):
            return self.decrease_size()
        else:
            return self.increase_size()

    def configGUI(self, start, stop, clear, txt="", color="blue"):
        if darwin:  # Leave Mac button colors alone - #44254.
            self.start_btn.config(state=start)
            self.stop_btn.config(state=stop)
            self.clear_btn.config(state=clear)
        else:
            self.start_btn.config(state=start,
                                  bg="#d00" if start == NORMAL else "#fca")
            self.stop_btn.config(state=stop,
                                 bg="#d00" if stop == NORMAL else "#fca")
            self.clear_btn.config(state=clear,
                                  bg="#d00" if clear == NORMAL else "#fca")
        self.output_lbl.config(text=txt, fg=color)

    def makeLoadDemoMenu(self, master):
        menu = Menu(master)

        for entry in getExampleEntries():
            def load(entry=entry):
                self.loadfile(entry)
            menu.add_command(label=entry, underline=0,
                             font=menufont, command=load)
        return menu

    def makeFontMenu(self, master):
        menu = Menu(master)
        menu.add_command(label="减小 (Ctrl+减号)", command=self.decrease_size,
                         font=menufont)
        menu.add_command(label="增大 (Ctrl+加号)", command=self.increase_size,
                         font=menufont)
        menu.add_separator()

        for size in font_sizes:
            def resize(size=size):
                self.set_txtsize(size)
            menu.add_command(label=str(size), underline=0,
                             font=menufont, command=resize)
        return menu

    def makeHelpMenu(self, master):
        menu = Menu(master)

        for help_label, help_file in help_entries:
            def show(help_label=help_label, help_file=help_file):
                view_text(self.root, help_label, help_file)
            menu.add_command(label=help_label, font=menufont, command=show)
        return menu

    def refreshCanvas(self):
        if self.dirty:
            self.screen.clear()
            self.dirty=False

    def loadfile(self, filename):
        self.clearCanvas()
        turtle.TurtleScreen._RUNNING = False
        modname = 'turtledemo.' + filename
        __import__(modname)
        self.module = sys.modules[modname]
        with open(self.module.__file__, 'r', encoding='utf-8') as f:
            chars = f.read()
        self.text.delete("1.0", "end")
        self.text.insert("1.0", chars)
        self.root.title(filename + " - 海龟画图演示")
        self.configGUI(NORMAL, DISABLED, DISABLED,
                       "点击开始按钮", "red")
        self.state = READY

    def startDemo(self):
        self.refreshCanvas()
        self.dirty = True
        turtle.TurtleScreen._RUNNING = True
        self.configGUI(DISABLED, NORMAL, DISABLED,
                       "演示正在运行中...", "black")
        self.screen.clear()
        self.screen.mode("standard")
        self.state = RUNNING

        try:
            result = self.module.main()
            if result == "EVENTLOOP":
                self.state = EVENTDRIVEN
            else:
                self.state = DONE
        except turtle.Terminator:
            if self.root is None:
                return
            self.state = DONE
            result = "已停止!"
        if self.state == DONE:
            self.configGUI(NORMAL, DISABLED, NORMAL,
                           result)
        elif self.state == EVENTDRIVEN:
            self.exitflag = True
            self.configGUI(DISABLED, NORMAL, DISABLED,
                           "请使用鼠标与键盘，点击停止按钮结束", "red")

    def clearCanvas(self):
        self.refreshCanvas()
        self.screen._delete("all")
        self.scanvas.config(cursor="")
        self.configGUI(NORMAL, DISABLED, DISABLED)

    def stopIt(self):
        if self.exitflag:
            self.clearCanvas()
            self.exitflag = False
            self.configGUI(NORMAL, DISABLED, DISABLED,
                           "已停止!", "red")
        turtle.TurtleScreen._RUNNING = False

    def _destroy(self):
        turtle.TurtleScreen._RUNNING = False
        self.root.destroy()
        self.root = None


def main():
    demo = DemoWindow()
    demo.root.mainloop()

if __name__ == '__main__':
    main()
